__author__ = 'Maruf Maniruzzaman'


PASSWORD_HMAC_SIGNATURE = "hmac:"
PASSWORD_COLUMN_NAME = "password"
USER_COOKIE_NAME = "usersecret"
USER_PLAINTEXT_COOKIE_NAME = "user"
IDENTITY_TYPE_FB_GRAPH = 'fb_graph'
IDENTITY_TYPE_GOOGLE_OAUTH2 = "google_oauth2"