import base64

file_data_list=[
{
'name': '/local_settings.py', 'data': base64.b64decode('aW1wb3J0IG9zCgoiIiIKWW91IHNob3VsZCBwdXQgYWxsIGRldmVsb3BtZW50IHNldHRpbmdzIGluIGhlcmUuCiIiIgoKI1RPRE86IE1ha2Ugc3VyZSB5b3UgbWF0Y2ggdGhpcyB3aXRoIHlvdXIgYW5ndWxhciBhcHBsaWNhdGlvbiBhbmQgY29weSB0aGlzIHRvIHNldHRpbmdzIGZpbGUgaW4gcHJvZHVjdGlvbgpTVEFUSUNfUEFUSCA9IG9zLnBhdGguam9pbihvcy5wYXRoLmRpcm5hbWUob3MucGF0aC5yZWFscGF0aChfX2ZpbGVfXykpLCAiYW5ndWxhci1zZWVkL2FwcCIpCklOREVYX0hUTUxfUEFUSCA9IG9zLnBhdGguam9pbihTVEFUSUNfUEFUSCwgImluZGV4Lmh0bWwiKQo=')
}]
